/* ---------------------------------------------------------------------------
 * Haskell interface to Maple
 * 
 * (c) 2000
 *     Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
 *     Hans-Wolfgang Loidl <hwloidl@cee.hw.ac.uk>
 *
 * $Id: maple.h,v 1.1 2000/11/08 14:13:48 schreine Exp $
 * -------------------------------------------------------------------------*/

/* startup and termination */
void mapleInit(void);
void mapleTerm(void);

/* ---------------------------------------------------------------------------
 * I/O functions
 * -------------------------------------------------------------------------*/

/* write len bytes starting at address to file */
void writeString(FILE *fd, char *addr, int len);

/* write len bytes starting at address to file, terminate by new line */
void writeLine(FILE *fd, char *addr, int len);

/* ----------------------------------------------------------------------------
 * types
 * ------------------------------------------------------------------------- */

/* the core result of a Maple call */
typedef struct
{
  int  n;     /* number of characters */
  char *addr; /* start of character buffer */
}
MapleVal;

/* the return value of Maple call */
typedef struct 
{
  int      ret; /* success code */
  MapleVal val; /* result value */
} 
MapleRet;

/* the return value of Maple call */
typedef struct 
{
  int ret; /* success code */
  int val; /* result value */
} 
MapleRetInt;

/* the return value of Maple call */
typedef struct 
{
  int  ret; /* success code */
  char val; /* result value */
} 
MapleRetChar;

/* the return value of Maple call */
typedef struct 
{
  int   ret; /* success code */
  int   len; /* length of string */ 
  char *str; /* character buffer */
} 
MapleRetString;

/* ---------------------------------------------------------------------------
 * type conversions
 * -------------------------------------------------------------------------*/

/* Haskell types to Maple types */
MapleRet *mapleInt(int);
MapleRet *mapleInteger(char *, int n);
MapleRet *mapleChar(char);
MapleRet *mapleExpr(char*, int n);

/* Maple types to Haskell Types */
MapleRetInt    *haskellInt(char *, int n);
MapleRetChar   *haskellChar(char *, int n);
MapleRetString *haskellString(char *, int n);

/* ---------------------------------------------------------------------------
 * Maple function evaluation
 *
 * mapleStartEval(n)
 * mapleOut(): name, arg1, ..., argn as separate lines
 * mapleEndEval() 
 * mapleResult() 
 *
 * mapleStartEvalN(n)
 * mapleOut(): name, arg1, ..., argn as separate lines
 * mapleEndEval() 
 * m = mapleResultN()
 * mapleResult() m times
 * ------------------------------------------------------------------------ */

/* start Maple function evaluation with n arguments and single result */
void mapleStartEval(int n);

/* handle of pipe to Maple */
FILE *mapleOut();

/* called after sending function name and arguments */
void mapleEndEval();

/* returns non-zero iff result is available */
int mapleResultAvailable();

/* returns result */
MapleRet *mapleResult();

/* start Maple function evaluation with n arguments and multiple results */
void mapleStartEvalN(int n);

/* return number of results */
int mapleResultN();

/* returns next result */
MapleRet *mapleNextResult();

/* ---------------------------------------------------------------------------
 * $Id: maple.h,v 1.1 2000/11/08 14:13:48 schreine Exp $
 * -------------------------------------------------------------------------*/
